@@ -8,6 +8,7 @@
 #include <string.h>
 #include <iostream>
 #include <sstream>
+#include <string_view>
 
 #include <zypp/ZYppFactory.h>
 #include <zypp/base/LogTools.h>
@@ -26,6 +27,7 @@
 #include "Zypper.h"
 
 #include "Summary.h"
+#include "utils/console.h"
 
 // Suppress all application related summary messages.
 // Summary messages produce lots of false reports about deleted applications. This
@@ -37,6 +39,138 @@
 
 // --------------------------------------------------------------------------
 
+extern ZYpp::Ptr God;
+namespace {
+  using ResolverOption = bool (zypp::Resolver::*)() const;
+
+  /** Hint if solver policy violations may be caused by --force-resolution */
+  std::string _resolverForceResolutionHint( ResolverOption option_r )
+  {
+    std::string ret;
+    if ( God->resolver()->forceResolve() && not ((*God->resolver()).*option_r)() ) {
+      ret = MSG_WARNINGString( " [--force-resolution]" ).str();
+    }
+    return ret;
+  }
+
+  std::string resolverForceResolutionHintDowngrade()
+  { return _resolverForceResolutionHint( &zypp::Resolver::allowDowngrade ); }
+
+  std::string resolverForceResolutionHintArchChange()
+  { return _resolverForceResolutionHint( &zypp::Resolver::allowArchChange ); }
+
+  std::string resolverForceResolutionHintVendorChange()
+  { return _resolverForceResolutionHint( &zypp::Resolver::allowVendorChange ); }
+
+
+  // --------------------------------------------------------------------------
+  struct SolvableSummaryItemFormater
+  {
+    SolvableSummaryItemFormater( ansi::Color color_r, bool withKind_r = false )
+    : _configHighlight { Zypper::instance().config().color_pkglistHighlight }
+    , _configAttr { Zypper::instance().config().color_pkglistHighlightAttribute }
+    , _contextColor { color_r }
+    , _withKind { withKind_r }
+    {
+      if ( _withKind )
+        _extractFn = &SolvableSummaryItemFormater::extractIdent;
+      if ( _configHighlight )
+        _formatFn = &SolvableSummaryItemFormater::formatHighlightFirst;
+      else if ( indeterminate(_configHighlight) )
+        _formatFn = &SolvableSummaryItemFormater::formatHighlightIfChanged;
+    }
+
+    template<class TSolv>
+    std::ostream & operator()( std::ostream & str, const sat::SolvableType<TSolv> & slv_r ) const
+    { return operator()( str, slv_r.satSolvable() ); }
+
+    std::ostream & operator()( std::ostream & str, sat::Solvable slv_r ) const
+    { return quoteIf( str, (this->*_extractFn)( slv_r ) ); }
+
+ private:
+    std::ostream & quoteIf( std::ostream & str, const std::string & txt_r ) const
+    {
+      if ( txt_r.find_first_of( " " ) == std::string::npos )
+        return (this->*_formatFn)( str, txt_r );
+      else {
+        static const HIGHLIGHTString quoteCh( "\"" );
+        return (this->*_formatFn)( str << quoteCh, txt_r ) << quoteCh;
+      }
+    }
+
+  private:
+    using ExtractFn = std::string (SolvableSummaryItemFormater::*)( sat::Solvable slv_r ) const;
+    using FormatFn = std::ostream & (SolvableSummaryItemFormater::*)( std::ostream &, const std::string & ) const;
+
+  private:
+    std::string extractName( sat::Solvable slv_r ) const
+    { return slv_r.name(); }
+
+    std::string extractIdent( sat::Solvable slv_r ) const
+    { return slv_r.ident().asString(); }
+
+  private:
+    std::ostream & formatPlain( std::ostream & str, const std::string & txt_r ) const
+    { return str << txt_r; }
+
+    std::ostream & formatHighlightFirst( std::ostream & str, const std::string & txt_r ) const
+    {
+      std::string_view txt { txt_r };
+      return str << ( _contextColor << _configAttr << txt.substr( 0, 1 ) ) << txt.substr( 1 ); }
+
+    std::ostream & formatHighlightIfChanged( std::ostream & str, const std::string & txt_r ) const
+    {
+      if ( txt_r[0] != _first ) {
+        _first = txt_r[0];
+        return formatHighlightFirst( str, txt_r );
+      }
+      return formatPlain( str, txt_r );
+    }
+
+   private:
+    TriBool _configHighlight;   ///< always/never or on a changed first char
+    ansi::Color _configAttr;    ///< highlight attribute from config
+    ansi::Color _contextColor;  ///< list color may be overwritten by _configAttr
+    bool _withKind = false;     ///< prefix names by kind
+
+    mutable char _first = '\0'; ///< for changed first char highlighting
+
+    ExtractFn _extractFn = &SolvableSummaryItemFormater::extractName;
+    FormatFn _formatFn = &SolvableSummaryItemFormater::formatPlain;
+  };
+
+  template <typename TContainer>  // sat::SolvableType
+  std::ostream & writeSolvableList( std::ostream & str, const TContainer & container_r, ansi::Color color_r, bool withKind_r=false )
+  {
+    constexpr unsigned indent = 2;
+    SolvableSummaryItemFormater format { color_r, withKind_r };
+
+    std::ostringstream s;
+    for ( const auto & slv : container_r ) {
+      format( s, slv ) << " "; // don't mind trailing WS, it's eaten in mbs_write_wrapped
+    }
+    mbs_write_wrapped( str, s.str(), indent, get_screen_width() );
+    return str << endl;
+  }
+
+  template <typename TContainer>  // std::pair<sat::SolvableType,std::string>
+  std::ostream & writeSolvableDetailsList( std::ostream & str, const TContainer & container_r, ansi::Color color_r, bool withKind_r=false )
+  {
+    constexpr unsigned indent = 2;
+    SolvableSummaryItemFormater format { color_r, withKind_r };
+
+
+    std::ostringstream s;
+    for ( const auto & pair : container_r ) {
+      format( s, pair.first ) << " " << pair.second << endl;
+    }
+    mbs_write_wrapped( str, s.str(), indent, get_screen_width() );
+    return str << endl;
+  }
+} // namespace
+
+// --------------------------------------------------------------------------
+
 bool Summary::ResPairNameCompare::operator()( const ResPair & p1, const ResPair & p2 ) const
 {
   int ret = ::strcoll( p1.second->name().c_str(), p2.second->name().c_str() );
@@ -47,11 +181,13 @@ bool Summary::ResPairNameCompare::operator()( const ResPair & p1, const ResPair
 
 // --------------------------------------------------------------------------
 
-Summary::Summary( const ResPool & pool, const ViewOptions options )
-: _viewop( options )
+Summary::Summary( const ResPool & pool, SummaryHints summaryHints, const ViewOptions options )
+: _summaryHints { std::move(summaryHints) }
+, _viewop( options )
 , _wrap_width( 80 )
 , _force_no_color( false )
 , _download_only( false )
+, _dryRun( false )
 {
   readPool( pool );
 }
@@ -178,8 +314,8 @@ void Summary::readPool( const ResPool & pool )
     for_( resit, it->second.begin(), it->second.end() )
     {
       ResObject::constPtr res(*resit);
-
       Package::constPtr pkg = asKind<Package>(res);
+
       if ( pkg )
       {
         switch ( pkg->vendorSupport() )
@@ -193,6 +329,8 @@ void Summary::readPool( const ResPool & pool )
           case VendorSupportACC:
             _supportNeedACC[res->kind()].insert( ResPair( nullptr, res ) );
             break;
+          case VendorSupportSuperseded:
+            _supportSuperseded[res->kind()].insert( ResPair( nullptr, res ) );
           default:
             // L1, L2 or L3 support are not reported
             break;
@@ -413,7 +551,7 @@ namespace
           // translator: '%1%' is a products name
           //             '%2%' is a command to call (like 'zypper dup')
           //             Both may contain whitespace and should be enclosed by quotes!
-          std::string ctcmsg( str::Format( _("Product '%1%' requires to be updated by calling '%2%'!") ) % obj_r->summary() % DEFAULTString("zypper dup") );
+          std::string ctcmsg( str::Format( _("Product '%1%' requires to be upgraded by calling '%2%'!") ) % obj_r->summary() % DEFAULTString("zypper dup") );
           // ^^^ summary() for products like in ResPair2Name
           //     DEFAULTString to prevent the command string from being colored in MSG_WARNINGString below
           tr_r.addDetail( MSG_WARNINGString( ctcmsg ) );
@@ -499,7 +637,7 @@ bool Summary::writeResolvableList( std::ostream & out,
   }
 
   Table t;
-  t.lineStyle(none);
+  t.lineStyle(TableLineStyle::none);
   t.margin(2);
   t.wrap(0);
 
@@ -752,7 +890,7 @@ void Summary::writeDowngraded( std::ostream & out )
 #endif
     label = str::form( label.c_str(), it->second.size() );
 
-    out << endl << ( ColorContext::NEGATIVE << label ) << endl;
+    out << endl << ( ColorContext::NEGATIVE << label ) << resolverForceResolutionHintDowngrade() << endl;
     writeResolvableList( out, it->second, ColorContext::NEGATIVE );
   }
 }
@@ -1160,7 +1298,7 @@ void Summary::writeChangedArch( std::ostream & out )
 #endif
     label = str::form( label.c_str(), it->second.size() );
 
-    out << endl << ( ColorContext::CHANGE << label ) << endl;
+    out << endl << ( ColorContext::CHANGE << label ) << resolverForceResolutionHintArchChange() << endl;
     writeResolvableList( out, it->second, ColorContext::CHANGE );
   }
 }
@@ -1206,7 +1344,7 @@ void Summary::writeChangedVendor( std::ostream & out )
 #endif
     label = str::form( label.c_str(), it->second.size() );
 
-    out << endl << ( ColorContext::CHANGE << label ) << endl;
+    out << endl << ( ColorContext::CHANGE << label ) << resolverForceResolutionHintVendorChange() << endl;
     writeResolvableList( out, it->second, ColorContext::CHANGE );
   }
 }
@@ -1267,6 +1405,46 @@ void Summary::writeSupportNeedACC( std::ostream & out )
   }
 }
 
+void Summary::writeSupportSuperseded( std::ostream & out )
+{
+  auto highlightColor = ColorContext::HIGHLIGHT;
+  auto supersedingAsString = []( Package::constPtr package_r ) -> std::string {
+    const std::pair<std::vector<IdString>,std::vector<std::string>> & supersededByItems { package_r->supersededByItems() };
+    const std::vector<IdString> & superseding { supersededByItems.first };
+    str::Str ret;
+    ret << _("was superseded by");
+    if ( not superseding.empty() )  // names resolved to existing packages
+      dumpRange( ret.stream(), superseding.begin(), superseding.end(), " ", "", ", ", "", "" );
+    else  // print at least the plain names
+      dumpRange( ret.stream(), supersededByItems.second.begin(), supersededByItems.second.end(), " ", "", ", ", "", "" );
+    return ret.str();
+  };
+
+  for_( it, _supportSuperseded.begin(), _supportSuperseded.end() )
+  {
+    // we only look at vendor support in packages
+    if ( it->first == ResKind::package ) {
+      std::string label = PL_(
+        "The following package was discontinued and has been superseded by a new package with a different name.",
+        "The following %d packages were discontinued and have been superseded by a new packages with different names.",
+        it->second.size() );
+      label = str::form( label.c_str(), it->second.size() );
+      std::string labelHint = PL_(
+        "The successor package should be installed as soon as possible to receive continued support:",
+        "The successor packages should be installed as soon as possible to receive continued support:",
+        it->second.size() );
+      out << endl << ( highlightColor << label << endl << labelHint ) << endl;
+
+      std::vector<std::pair<sat::Solvable,std::string>> itemsdetails;
+      for ( const auto & pair : it->second ) {
+        Package::constPtr package = pair.second->asKind<Package>();
+        itemsdetails.push_back( std::make_pair( package->satSolvable(), supersedingAsString( package ) ) );
+      }
+      writeSolvableDetailsList( out, itemsdetails, highlightColor );
+    }
+  }
+}
+
 // --------------------------------------------------------------------------
 
 void Summary::writeNotUpdated( std::ostream & out )
@@ -1377,6 +1555,15 @@ void Summary::writeRebootNeeded( std::ostream & out )
   checkAndPrintRebootNeeded( ResKind::package );
 }
 
+void Summary::writeAutoResolved( std::ostream & out )
+{
+  if ( _summaryHints.haveSkippedPatches() ) {
+    std::string label { _("Skipped needed patches which do not apply without conflict:") };
+    out << endl << ( ColorContext::MSG_WARNING << label << " [-skip-not-applicable-patches]" ) << endl;
+    writeSolvableList( out, _summaryHints.skippedPatchesSet(), ColorContext::MSG_WARNING );
+  }
+}
+
 // --------------------------------------------------------------------------
 
 void Summary::writeDownloadAndInstalledSizeSummary( std::ostream & out )
@@ -1594,11 +1781,13 @@ void Summary::dumpTo( std::ostream & out )
   writeChangedVendor(out);
   writeNewlyInstalled( out );
   writeRemoved( out );
+  writeAutoResolved( out );
   if ( _viewop & SHOW_UNSUPPORTED )
   {
     writeSupportUnknown( out );
     writeSupportUnsupported( out );
     writeSupportNeedACC( out );
+    writeSupportSuperseded( out );
   }
   writeRebootNeeded( out );
   writePackageCounts( out );
@@ -1609,7 +1798,15 @@ void Summary::dumpTo( std::ostream & out )
     Zypper::instance().out().notePar( 4, _("Package manager restart required. (Run this command once again after the update stack got updated)") );
   }
   if ( showNeedRebootHInt() )
-  {   Zypper::instance().out().notePar( 4, _("System reboot required.") ); }
+  {
+    if ( _download_only || _dryRun ) {
+      std::string_view opt { _dryRun ? "--dry-run" : "--download-only" };
+      // translator: %1% is an option string like  "--dry-run"
+      Zypper::instance().out().notePar( 4, str::Format( _("%1% is set, otherwise a system reboot would be required.") ) % opt );
+    } else {
+      Zypper::instance().out().notePar( 4, _("System reboot required.") );
+    }
+  }
 
   if ( !_ctc.empty() )
   {
@@ -1624,7 +1821,7 @@ void Summary::dumpTo( std::ostream & out )
     // # zypper up
     // ...
     //    Consider to cancel:
-    //    Product 'opennSUSE Tumbleweed' requires to be updated by calling 'zypper dup'!
+    //    Product 'opennSUSE Tumbleweed' requires to be upgraded by calling 'zypper dup'!
     // Continue? [y/n/...? shows all options] (y):
     Zypper::instance().out().taggedPar( 4, MSG_WARNINGString(_("Consider to cancel:") ), s );
   }
