@@ -17,9 +17,25 @@
 #include <zypp/base/DefaultIntegral.h>
 #include <zypp/ResObject.h>
 #include <zypp/ResPool.h>
+#include "utils/ansi.h"
+
+/// \brief Information collected in SolveAndCommit which is to be shown in the summary.
+struct SummaryHints
+{
+  void clear() { *this = SummaryHints(); }
+
+  bool haveSkippedPatches() const
+  { return _skippedPatches && not _skippedPatches->empty(); }
+
+  std::set<PoolItem> & skippedPatchesSet()
+  { if ( not _skippedPatches ) _skippedPatches = std::set<PoolItem>(); return *_skippedPatches; }
+
+private:
+  std::optional<std::set<PoolItem>> _skippedPatches; ///< --skip-not-applicable-patches
+};
 
 
-class Summary : private base::NonCopyable
+class Summary : private zypp::base::NonCopyable
 {
 public:
   /**
@@ -27,13 +43,13 @@ public:
    *        usually only 'second' is intialized, the 'first' var is only
    *        required to track upgrades, first is the removed and second the installed package
    */
-  typedef std::pair<ResObject::constPtr, ResObject::constPtr> ResPair;
+  typedef std::pair<zypp::ResObject::constPtr, zypp::ResObject::constPtr> ResPair;
   struct ResPairNameCompare
   {
     inline bool operator()( const ResPair & p1, const ResPair & p2 ) const;
   };
   typedef std::set<ResPair, ResPairNameCompare> ResPairSet;
-  typedef std::map<ResKind, ResPairSet> KindToResPairSet;
+  typedef std::map<zypp::ResKind, ResPairSet> KindToResPairSet;
 
   enum _view_options
   {
@@ -60,7 +76,7 @@ public:
   typedef enum _view_options ViewOptions;
 
 public:
-  Summary( const ResPool & pool, const ViewOptions options = DEFAULT );
+  Summary( const zypp::ResPool & pool, SummaryHints summaryHints, const ViewOptions options = DEFAULT );
   ~Summary() {}
 
   void setViewOptions( const ViewOptions options )	{ _viewop = options; }
@@ -71,6 +87,7 @@ public:
   bool hasViewOption( const ViewOptions option ) const  { return _viewop & option; }
   void setForceNoColor( bool value = true )		{ _force_no_color = value; }
   void setDownloadOnly( bool value = true )		{ _download_only = value; }
+  void setDryRun( bool value = true )			{ _dryRun = value; }
 
   void writeNewlyInstalled( std::ostream & out );
   void writeRemoved( std::ostream & out );
@@ -84,11 +101,13 @@ public:
   void writeSupportUnknown( std::ostream & out );
   void writeSupportUnsupported( std::ostream & out );
   void writeSupportNeedACC( std::ostream & out );
+  void writeSupportSuperseded( std::ostream & out );
   void writeNotUpdated( std::ostream & out );
   void writePackageCounts( std::ostream & out );
   void writeDownloadAndInstalledSizeSummary( std::ostream & out );
   void writeLocked( std::ostream & out );
   void writeRebootNeeded( std::ostream & out );
+  void writeAutoResolved( std::ostream & out );
 
   unsigned packagesToGetAndInstall() const		{ return _inst_pkg_total; }
 
@@ -98,9 +117,9 @@ public:
   unsigned packagesToReInstall() const;
   unsigned packagesToRemove() const;
 
-  const ByteCount & toDownload() const		{ return _todownload; }
-  const ByteCount & inCache() const		{ return _incache; }
-  const ByteCount & installedSizeChange() const	{ return _inst_size_change; }
+  const zypp::ByteCount & toDownload() const		{ return _todownload; }
+  const zypp::ByteCount & inCache() const		{ return _incache; }
+  const zypp::ByteCount & installedSizeChange() const	{ return _inst_size_change; }
 
   /** The exposed needMachineReboot value causing ZYPPER_EXIT_INF_REBOOT_NEEDED considers patches only (zypper#237)
    * Packages cause a summary hint but will not lead to a non-zero return value.
@@ -115,7 +134,7 @@ public:
   void dumpAsXmlTo( std::ostream & out );
 
 private:
-  void readPool( const ResPool & pool );
+  void readPool( const zypp::ResPool & pool );
 
   bool writeResolvableList( std::ostream & out, const ResPairSet & resolvables, ansi::Color = ansi::Color::nocolor(), unsigned maxEntries_r = 0U, bool withKind_r = false );
   bool writeResolvableList( std::ostream & out, const ResPairSet & resolvables, unsigned maxEntries_r, bool withKind_r = false )
@@ -123,24 +142,26 @@ private:
 
   void writeXmlResolvableList( std::ostream & out, const KindToResPairSet & resolvables );
 
-  void collectInstalledRecommends( const ResObject::constPtr & obj );
+  void collectInstalledRecommends( const zypp::ResObject::constPtr & obj );
 
   bool showNeedRestartHint() const;
   bool showNeedRebootHInt() const;
 
 private:
+  SummaryHints _summaryHints;   ///< additional info collected during dependency resolution
   ViewOptions _viewop;
   mutable unsigned _wrap_width;
   bool _force_no_color;
   bool _download_only;
+  bool _dryRun;
 
   bool _need_reboot_patch;	// need_reboot caused by a patch
   bool _need_reboot_nonpatch;	// need_reboot caused by something not a patch
   bool _need_restart;		// by now patch specific
 
-  ByteCount _todownload;
-  ByteCount _incache;
-  ByteCount _inst_size_change;
+  zypp::ByteCount _todownload;
+  zypp::ByteCount _incache;
+  zypp::ByteCount _inst_size_change;
 
   // STATS
 
@@ -161,6 +182,7 @@ private:
   KindToResPairSet _supportUnknown;
   KindToResPairSet _supportUnsupported;	///< known to be unsupported
   KindToResPairSet _supportNeedACC;	///< need additional contract
+  KindToResPairSet _supportSuperseded;  ///< discontinued and superseded by a new package
   /** Patches and Packages which require a reboot */
   KindToResPairSet _rebootNeeded;
 
